import React from 'react';
import ReactDOM from 'react-dom/client';
import { RouterProvider, createBrowserRouter } from 'react-router-dom';
import App from './App';
import './index.css';

// WebGL capability check
const checkWebGLSupport = () => {
  const canvas = document.createElement('canvas');
  const gl = canvas.getContext('webgl') || canvas.getContext('experimental-webgl');
  
  if (!gl) {
    console.warn('WebGL not supported. Using fallback rendering.');
    document.documentElement.classList.add('no-webgl');
    return false;
  }

  // Check for required extensions
  const requiredExtensions = ['OES_texture_float', 'WEBGL_depth_texture'];
  const missingExtensions = requiredExtensions.filter(ext => !(gl as WebGLRenderingContext).getExtension(ext));
  
  if (missingExtensions.length > 0) {
    console.warn(`Missing WebGL extensions: ${missingExtensions.join(', ')}`);
    document.documentElement.classList.add('limited-webgl');
    return false;
  }

  // Check for high-performance GPU
  const debugInfo = (gl as WebGLRenderingContext).getExtension('WEBGL_debug_renderer_info');
  if (debugInfo) {
    const renderer = (gl as WebGLRenderingContext).getParameter(debugInfo.UNMASKED_RENDERER_WEBGL);
    const isHighPerformance = /(nvidia|amd|radeon|intel)/i.test(renderer);
    if (!isHighPerformance) {
      console.warn('Low performance GPU detected. Some features may be limited.');
      document.documentElement.classList.add('low-performance-gpu');
    }
  }

  return true;
};

// Performance monitoring
const setupPerformanceMonitoring = () => {
  if (process.env.NODE_ENV === 'development') {
    const reportPerformance = () => {
      const navEntry = performance.getEntriesByType('navigation')[0] as PerformanceNavigationTiming;
      const paintEntry = performance.getEntriesByType('paint')[0];
      
      console.log('Page Load Metrics:', {
        'DOM Content Loaded': `${navEntry.domContentLoadedEventEnd}ms`,
        'First Paint': `${paintEntry?.startTime}ms`,
        'Load Complete': `${navEntry.loadEventEnd}ms`
      });
    };

    window.addEventListener('load', reportPerformance);
  }
};

// Error boundary for WebGL components
class WebGLErrorBoundary extends React.Component<{ children: React.ReactNode }> {
  state = { hasError: false };

  static getDerivedStateFromError() {
    return { hasError: true };
  }

  componentDidCatch(error: Error) {
    console.error('WebGL Error:', error);
    document.documentElement.classList.add('webgl-error');
  }

  render() {
    if (this.state.hasError) {
      return null; // Render nothing on error, letting fallbacks take over
    }
    return this.props.children;
  }
}

// Initialize app
const initializeApp = () => {
  checkWebGLSupport();
  setupPerformanceMonitoring();

  // Apply any necessary polyfills or patches
  if (!window.requestIdleCallback) {
    const requestIdleCallback = (callback: IdleRequestCallback): number => {
      const startTime = Date.now();
      return setTimeout(() => {
        callback({
          didTimeout: false,
          timeRemaining: () => Math.max(0, 50 - (Date.now() - startTime))
        });
      }, 1) as unknown as number;
    };

    const cancelIdleCallback = (id: number): void => {
      clearTimeout(id);
    };

    window.requestIdleCallback = requestIdleCallback;
    window.cancelIdleCallback = cancelIdleCallback;
  }

  // Create root and render app
  const root = ReactDOM.createRoot(document.getElementById('root') as HTMLElement);
  
  // Create router with improved handling for trailing slashes
  const router = createBrowserRouter([
    {
      path: '/*',
      element: <App />,
    }
  ], {
    // This ensures consistent behavior with or without trailing slashes
    basename: '/'
  });

  // Add global error handler
  window.addEventListener('error', (event) => {
    console.error('Global error caught:', event.error);
    // You could send this to a monitoring service in production
  });

  // Add unhandled promise rejection handler
  window.addEventListener('unhandledrejection', (event) => {
    console.error('Unhandled promise rejection:', event.reason);
    // You could send this to a monitoring service in production
  });

  root.render(
    <React.StrictMode>
      <WebGLErrorBoundary>
        <RouterProvider router={router} />
      </WebGLErrorBoundary>
    </React.StrictMode>
  );
};

initializeApp();
